@@ -42,7 +42,7 @@ create_boundingbox_tree(const dolfinx::mesh::Mesh<U>& mesh,
   std::vector<std::int32_t> cells = dolfinx::mesh::compute_incident_entities(
       *mesh.topology(), facets, dim, tdim);
 
-  dolfinx::geometry::BoundingBoxTree<U> bb_tree(mesh, tdim, cells, padding);
+  dolfinx::geometry::BoundingBoxTree<U> bb_tree(mesh, tdim, padding, cells);
   return bb_tree;
 }
 
@@ -363,7 +363,7 @@ mpc_data<T> create_contact_slip_condition(
 {
 
   dolfinx::common::Timer timer("~MPC: Create slip constraint");
-  std::shared_ptr<const mesh::Mesh<U>> mesh = V.mesh();
+  std::shared_ptr<const dolfinx::mesh::Mesh<U>> mesh = V.mesh();
   MPI_Comm comm = mesh->comm();
   int rank = -1;
   MPI_Comm_rank(comm, &rank);
@@ -469,7 +469,7 @@ mpc_data<T> create_contact_slip_condition(
                                                 slave_coordinates, eps2);
 
     auto [basis, basis_shape] = dolfinx_mpc::evaluate_basis_functions<U>(
-        V, slave_coordinates, local_cell_collisions);
+        V, slave_coordinates, local_cell_collisions, eps2);
     assert(basis_shape.back() == 1);
     MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
         U, MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
@@ -542,8 +542,8 @@ mpc_data<T> create_contact_slip_condition(
   const std::size_t out_collision_slaves = slave_indices_remote.size();
   std::vector<std::int32_t> num_slaves_recv(indegree + 1);
   MPI_Neighbor_allgather(
-      &out_collision_slaves, 1, dolfinx::MPI::mpi_type<std::int32_t>(),
-      num_slaves_recv.data(), 1, dolfinx::MPI::mpi_type<std::int32_t>(),
+      &out_collision_slaves, 1, dolfinx::MPI::mpi_t<std::int32_t>,
+      num_slaves_recv.data(), 1, dolfinx::MPI::mpi_t<std::int32_t>,
       neighborhood_comms[0]);
   num_slaves_recv.pop_back();
 
@@ -555,9 +555,9 @@ mpc_data<T> create_contact_slip_condition(
   // Send data to neighbors and receive data
   std::vector<std::int32_t> recv_rems(disp.back());
   MPI_Neighbor_allgatherv(send_rems.data(), (int)send_rems.size(),
-                          dolfinx::MPI::mpi_type<std::int32_t>(),
-                          recv_rems.data(), num_slaves_recv.data(), disp.data(),
-                          dolfinx::MPI::mpi_type<std::int32_t>(),
+                          dolfinx::MPI::mpi_t<std::int32_t>, recv_rems.data(),
+                          num_slaves_recv.data(), disp.data(),
+                          dolfinx::MPI::mpi_t<std::int32_t>,
                           neighborhood_comms[0]);
 
   // Multiply recv size by three to accommodate vector coordinates and
@@ -574,14 +574,14 @@ mpc_data<T> create_contact_slip_condition(
   // Send slave normal and coordinate to neighbors
   std::vector<U> recv_coords(disp.back() * 3);
   MPI_Neighbor_allgatherv(coordinates_send.data(), (int)coordinates_send.size(),
-                          dolfinx::MPI::mpi_type<U>(), recv_coords.data(),
+                          dolfinx::MPI::mpi_t<U>, recv_coords.data(),
                           num_slaves_recv3.data(), disp3.data(),
-                          dolfinx::MPI::mpi_type<U>(), neighborhood_comms[0]);
+                          dolfinx::MPI::mpi_t<U>, neighborhood_comms[0]);
   std::vector<U> slave_normals(disp.back() * 3);
   MPI_Neighbor_allgatherv(normals_send.data(), (int)normals_send.size(),
-                          dolfinx::MPI::mpi_type<U>(), slave_normals.data(),
+                          dolfinx::MPI::mpi_t<U>, slave_normals.data(),
                           num_slaves_recv3.data(), disp3.data(),
-                          dolfinx::MPI::mpi_type<U>(), neighborhood_comms[0]);
+                          dolfinx::MPI::mpi_t<U>, neighborhood_comms[0]);
 
   int err0 = MPI_Comm_free(&neighborhood_comms[0]);
   dolfinx::MPI::check_error(comm, err0);
@@ -593,7 +593,7 @@ mpc_data<T> create_contact_slip_condition(
         = dolfinx_mpc::find_local_collisions<U>(*mesh, bb_tree, recv_coords,
                                                 eps2);
     auto [recv_basis_values, shape] = dolfinx_mpc::evaluate_basis_functions<U>(
-        V, recv_coords, remote_cell_collisions);
+        V, recv_coords, remote_cell_collisions, eps2);
     MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
         U, MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
         basis_span(recv_basis_values.data(), shape[0], shape[1]);
@@ -670,31 +670,30 @@ mpc_data<T> create_contact_slip_condition(
   std::vector<std::int32_t> remote_colliding_offsets(inc_disp_offsets.back());
   MPI_Ineighbor_alltoallv(
       offsets_remote.data(), num_out_offsets.data(), send_disp_offsets.data(),
-      dolfinx::MPI::mpi_type<std::int32_t>(), remote_colliding_offsets.data(),
+      dolfinx::MPI::mpi_t<std::int32_t>, remote_colliding_offsets.data(),
       num_inc_offsets.data(), inc_disp_offsets.data(),
-      dolfinx::MPI::mpi_type<std::int32_t>(), neighborhood_comms[1],
-      &requests[0]);
+      dolfinx::MPI::mpi_t<std::int32_t>, neighborhood_comms[1], &requests[0]);
   // Receive colliding masters and relevant data from other processor
   std::vector<std::int64_t> remote_colliding_masters(disp_inc_masters.back());
   MPI_Ineighbor_alltoallv(
       remote_data.masters.data(), num_collision_masters.data(),
-      send_disp_masters.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
+      send_disp_masters.data(), dolfinx::MPI::mpi_t<std::int64_t>,
       remote_colliding_masters.data(), inc_num_collision_masters.data(),
-      disp_inc_masters.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
+      disp_inc_masters.data(), dolfinx::MPI::mpi_t<std::int64_t>,
       neighborhood_comms[1], &requests[1]);
   std::vector<T> remote_colliding_coeffs(disp_inc_masters.back());
   MPI_Ineighbor_alltoallv(
       remote_data.coeffs.data(), num_collision_masters.data(),
-      send_disp_masters.data(), dolfinx::MPI::mpi_type<T>(),
+      send_disp_masters.data(), dolfinx::MPI::mpi_t<T>,
       remote_colliding_coeffs.data(), inc_num_collision_masters.data(),
-      disp_inc_masters.data(), dolfinx::MPI::mpi_type<T>(),
-      neighborhood_comms[1], &requests[2]);
+      disp_inc_masters.data(), dolfinx::MPI::mpi_t<T>, neighborhood_comms[1],
+      &requests[2]);
   std::vector<std::int32_t> remote_colliding_owners(disp_inc_masters.back());
   MPI_Ineighbor_alltoallv(
       remote_data.owners.data(), num_collision_masters.data(),
-      send_disp_masters.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
+      send_disp_masters.data(), dolfinx::MPI::mpi_t<std::int32_t>,
       remote_colliding_owners.data(), inc_num_collision_masters.data(),
-      disp_inc_masters.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
+      disp_inc_masters.data(), dolfinx::MPI::mpi_t<std::int32_t>,
       neighborhood_comms[1], &requests[3]);
 
   // Wait for offsets to be sent
@@ -859,7 +858,7 @@ mpc_data<T> create_contact_slip_condition(
   // Distribute ghost data
   dolfinx_mpc::mpc_data ghost_data = dolfinx_mpc::distribute_ghost_data<T>(
       local_slaves, local_masters, local_coeffs, local_owners,
-      num_masters_per_slave, *imap, block_size);
+      num_masters_per_slave, imap, block_size);
 
   // Add ghost data to existing arrays
   const std::vector<std::int32_t>& ghost_slaves = ghost_data.slaves;
@@ -899,13 +898,15 @@ mpc_data<T> create_contact_slip_condition(
 /// @param[in] master_marker Tag for the other interface
 /// @param[in] eps2 The tolerance for the squared distance to be considered a
 /// collision
-/// @param[in] allow_missing_masters If true, the function will not throw an error if
-/// a degree of freedom in the closure of the master entities does not have a corresponding set of slave degrees of freedom.
+/// @param[in] allow_missing_masters If true, the function will not throw an
+/// error if a degree of freedom in the closure of the master entities does not
+/// have a corresponding set of slave degrees of freedom.
 template <typename T, std::floating_point U>
 mpc_data<T> create_contact_inelastic_condition(
     const dolfinx::fem::FunctionSpace<U>& V,
     dolfinx::mesh::MeshTags<std::int32_t> meshtags, std::int32_t slave_marker,
-    std::int32_t master_marker, const U eps2 = 1e-20, bool allow_missing_masters = false)
+    std::int32_t master_marker, const U eps2 = 1e-20,
+    bool allow_missing_masters = false)
 {
   dolfinx::common::Timer timer("~MPC: Inelastic condition");
 
@@ -1005,7 +1006,7 @@ mpc_data<T> create_contact_inelastic_condition(
         = dolfinx_mpc::find_local_collisions<U>(*V.mesh(), bb_tree,
                                                 slave_coordinates, eps2);
     auto [basis_values, basis_shape] = dolfinx_mpc::evaluate_basis_functions<U>(
-        V, slave_coordinates, colliding_cells);
+        V, slave_coordinates, colliding_cells, eps2);
     assert(basis_shape.back() == 1);
     MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
         U, MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
@@ -1109,51 +1110,56 @@ mpc_data<T> create_contact_inelastic_condition(
     {
       int counter = 0;
       std::ranges::for_each(
-        local_blocks,
-        [block_size, tdim, &masters_out, &local_masters, &coeffs_out,
-         &local_coeffs, &offsets_out, &slaves, &counter, &collision_to_local](const std::int32_t block)
-        {
-          if (std::find(collision_to_local.begin(), collision_to_local.end(), counter++) != collision_to_local.end())
-          {
-            std::cout << "Skipping block " << block << " without local collision." << counter << std::endl;
-            return; // Skip blocks without local collision
-          }
-          else
+          local_blocks,
+          [block_size, tdim, &masters_out, &local_masters, &coeffs_out,
+           &local_coeffs, &offsets_out, &slaves, &counter,
+           &collision_to_local](const std::int32_t block)
           {
+            if (std::find(collision_to_local.begin(), collision_to_local.end(),
+                          counter++)
+                != collision_to_local.end())
+            {
+              std::cout << "Skipping block " << block
+                        << " without local collision." << counter << std::endl;
+              return; // Skip blocks without local collision
+            }
+            else
+            {
 
-          for (std::int32_t j = 0; j < block_size; ++j)
-          {
-            const std::int32_t slave = block * block_size + j;
-            slaves.push_back(slave);
-            masters_out.insert(masters_out.end(), local_masters[slave].begin(),
-                               local_masters[slave].end());
-            coeffs_out.insert(coeffs_out.end(), local_coeffs[slave].begin(),
-                              local_coeffs[slave].end());
-            offsets_out.push_back((std::int32_t)masters_out.size());
-          }
-        }
-        });
-
+              for (std::int32_t j = 0; j < block_size; ++j)
+              {
+                const std::int32_t slave = block * block_size + j;
+                slaves.push_back(slave);
+                masters_out.insert(masters_out.end(),
+                                   local_masters[slave].begin(),
+                                   local_masters[slave].end());
+                coeffs_out.insert(coeffs_out.end(), local_coeffs[slave].begin(),
+                                  local_coeffs[slave].end());
+                offsets_out.push_back((std::int32_t)masters_out.size());
+              }
+            }
+          });
     }
     else
     {
       std::ranges::for_each(
-        local_blocks,
-        [block_size, tdim, &masters_out, &local_masters, &coeffs_out,
-         &local_coeffs, &offsets_out, &slaves](const std::int32_t block)
-        {
-          for (std::int32_t j = 0; j < block_size; ++j)
+          local_blocks,
+          [block_size, tdim, &masters_out, &local_masters, &coeffs_out,
+           &local_coeffs, &offsets_out, &slaves](const std::int32_t block)
           {
-            const std::int32_t slave = block * block_size + j;
-            slaves.push_back(slave);
-            masters_out.insert(masters_out.end(), local_masters[slave].begin(),
-                               local_masters[slave].end());
-            coeffs_out.insert(coeffs_out.end(), local_coeffs[slave].begin(),
-                              local_coeffs[slave].end());
-            offsets_out.push_back((std::int32_t)masters_out.size());
-          }
-        });
-      }
+            for (std::int32_t j = 0; j < block_size; ++j)
+            {
+              const std::int32_t slave = block * block_size + j;
+              slaves.push_back(slave);
+              masters_out.insert(masters_out.end(),
+                                 local_masters[slave].begin(),
+                                 local_masters[slave].end());
+              coeffs_out.insert(coeffs_out.end(), local_coeffs[slave].begin(),
+                                local_coeffs[slave].end());
+              offsets_out.push_back((std::int32_t)masters_out.size());
+            }
+          });
+    }
     std::vector<std::int32_t> owners(masters_out.size());
     std::ranges::fill(owners, 0);
     mpc.slaves = slaves;
@@ -1175,8 +1181,8 @@ mpc_data<T> create_contact_inelastic_condition(
   const auto num_colliding_blocks = (int)blocks_wo_local_collision.size();
   std::vector<std::int32_t> num_slave_blocks(indegree + 1);
   MPI_Neighbor_allgather(
-      &num_colliding_blocks, 1, dolfinx::MPI::mpi_type<std::int32_t>(),
-      num_slave_blocks.data(), 1, dolfinx::MPI::mpi_type<std::int32_t>(),
+      &num_colliding_blocks, 1, dolfinx::MPI::mpi_t<std::int32_t>,
+      num_slave_blocks.data(), 1, dolfinx::MPI::mpi_t<std::int32_t>,
       neighborhood_comms[0]);
   num_slave_blocks.pop_back();
 
@@ -1189,9 +1195,9 @@ mpc_data<T> create_contact_inelastic_condition(
   std::vector<std::int64_t> remote_slave_blocks(disp.back());
   MPI_Neighbor_allgatherv(
       blocks_wo_local_collision.data(), num_colliding_blocks,
-      dolfinx::MPI::mpi_type<std::int64_t>(), remote_slave_blocks.data(),
-      num_slave_blocks.data(), disp.data(),
-      dolfinx::MPI::mpi_type<std::int64_t>(), neighborhood_comms[0]);
+      dolfinx::MPI::mpi_t<std::int64_t>, remote_slave_blocks.data(),
+      num_slave_blocks.data(), disp.data(), dolfinx::MPI::mpi_t<std::int64_t>,
+      neighborhood_comms[0]);
 
   // Multiply recv size by three to accommodate block coordinates
   std::vector<std::int32_t> num_block_coordinates(indegree);
@@ -1203,11 +1209,10 @@ mpc_data<T> create_contact_inelastic_condition(
 
   // Send slave coordinates to neighbors
   std::vector<U> recv_coords(disp.back() * 3);
-  MPI_Neighbor_allgatherv(distribute_coordinates.data(),
-                          (int)distribute_coordinates.size(),
-                          dolfinx::MPI::mpi_type<U>(), recv_coords.data(),
-                          num_block_coordinates.data(), coordinate_disp.data(),
-                          dolfinx::MPI::mpi_type<U>(), neighborhood_comms[0]);
+  MPI_Neighbor_allgatherv(
+      distribute_coordinates.data(), (int)distribute_coordinates.size(),
+      dolfinx::MPI::mpi_t<U>, recv_coords.data(), num_block_coordinates.data(),
+      coordinate_disp.data(), dolfinx::MPI::mpi_t<U>, neighborhood_comms[0]);
 
   int err0 = MPI_Comm_free(&neighborhood_comms[0]);
   dolfinx::MPI::check_error(comm, err0);
@@ -1228,7 +1233,7 @@ mpc_data<T> create_contact_inelastic_condition(
         = dolfinx_mpc::find_local_collisions<U>(*V.mesh(), bb_tree, recv_coords,
                                                 eps2);
     auto [basis, basis_shape] = dolfinx_mpc::evaluate_basis_functions<U>(
-        V, recv_coords, remote_cell_collisions);
+        V, recv_coords, remote_cell_collisions, eps2);
     assert(basis_shape.back() == 1);
     MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
         const U, MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
@@ -1375,9 +1380,9 @@ mpc_data<T> create_contact_inelastic_condition(
       disp_inc_slave_blocks.back());
   MPI_Neighbor_alltoallv(
       found_slave_blocks.data(), num_found_slave_blocks.data(),
-      send_disp_slave_blocks.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
+      send_disp_slave_blocks.data(), dolfinx::MPI::mpi_t<std::int64_t>,
       remote_colliding_blocks.data(), inc_num_found_slave_blocks.data(),
-      disp_inc_slave_blocks.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
+      disp_inc_slave_blocks.data(), dolfinx::MPI::mpi_t<std::int64_t>,
       neighborhood_comms[1]);
   std::vector<std::int32_t> recv_blocks_as_local(
       remote_colliding_blocks.size());
@@ -1386,31 +1391,30 @@ mpc_data<T> create_contact_inelastic_condition(
       disp_inc_slave_blocks.back());
   MPI_Neighbor_alltoallv(
       offset_for_blocks.data(), num_found_slave_blocks.data(),
-      send_disp_slave_blocks.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
+      send_disp_slave_blocks.data(), dolfinx::MPI::mpi_t<std::int32_t>,
       remote_colliding_offsets.data(), inc_num_found_slave_blocks.data(),
-      disp_inc_slave_blocks.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
+      disp_inc_slave_blocks.data(), dolfinx::MPI::mpi_t<std::int32_t>,
       neighborhood_comms[1]);
   // Receive colliding masters and relevant data from other processor
   std::vector<std::int64_t> remote_colliding_masters(disp_inc_masters.back());
   MPI_Neighbor_alltoallv(
       found_masters.data(), num_collision_masters.data(),
-      send_disp_masters.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
+      send_disp_masters.data(), dolfinx::MPI::mpi_t<std::int64_t>,
       remote_colliding_masters.data(), num_inc_masters.data(),
-      disp_inc_masters.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
+      disp_inc_masters.data(), dolfinx::MPI::mpi_t<std::int64_t>,
       neighborhood_comms[1]);
   std::vector<T> remote_colliding_coeffs(disp_inc_masters.back());
   MPI_Neighbor_alltoallv(found_coefficients.data(),
                          num_collision_masters.data(), send_disp_masters.data(),
-                         dolfinx::MPI::mpi_type<T>(),
-                         remote_colliding_coeffs.data(), num_inc_masters.data(),
-                         disp_inc_masters.data(), dolfinx::MPI::mpi_type<T>(),
-                         neighborhood_comms[1]);
+                         dolfinx::MPI::mpi_t<T>, remote_colliding_coeffs.data(),
+                         num_inc_masters.data(), disp_inc_masters.data(),
+                         dolfinx::MPI::mpi_t<T>, neighborhood_comms[1]);
   std::vector<std::int32_t> remote_colliding_owners(disp_inc_masters.back());
   MPI_Neighbor_alltoallv(
       found_owners.data(), num_collision_masters.data(),
-      send_disp_masters.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
+      send_disp_masters.data(), dolfinx::MPI::mpi_t<std::int32_t>,
       remote_colliding_owners.data(), num_inc_masters.data(),
-      disp_inc_masters.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
+      disp_inc_masters.data(), dolfinx::MPI::mpi_t<std::int32_t>,
       neighborhood_comms[1]);
 
   // Create receive displacement of data per slave block
@@ -1431,9 +1435,9 @@ mpc_data<T> create_contact_inelastic_condition(
   std::vector<std::int32_t> block_dofs_recv(inc_block_disp.back());
   MPI_Neighbor_alltoallv(
       offsets_in_blocks.data(), num_found_blocks.data(), send_block_disp.data(),
-      dolfinx::MPI::mpi_type<std::int32_t>(), block_dofs_recv.data(),
+      dolfinx::MPI::mpi_t<std::int32_t>, block_dofs_recv.data(),
       recv_num_found_blocks.data(), inc_block_disp.data(),
-      dolfinx::MPI::mpi_type<std::int32_t>(), neighborhood_comms[1]);
+      dolfinx::MPI::mpi_t<std::int32_t>, neighborhood_comms[1]);
 
   int err1 = MPI_Comm_free(&neighborhood_comms[1]);
   dolfinx::MPI::check_error(comm, err1);
@@ -1503,7 +1507,8 @@ mpc_data<T> create_contact_inelastic_condition(
   {
     for (int d = 0; d < block_size; d++)
     {
-      if ((local_masters[block * block_size + d].empty()) and (!allow_missing_masters))
+      if ((local_masters[block * block_size + d].empty())
+          and (!allow_missing_masters))
       {
         throw std::runtime_error(
             "No masters found on contact surface for slave. Please run in "
@@ -1637,19 +1642,19 @@ mpc_data<T> create_contact_inelastic_condition(
                    disp_send_ghost_slaves.begin() + 1);
 
   std::vector<std::int64_t> in_ghost_slaves(disp_recv_ghost_slaves.back());
-  MPI_Neighbor_alltoallv(
-      out_ghost_slaves.data(), num_send_slaves.data(),
-      disp_send_ghost_slaves.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
-      in_ghost_slaves.data(), inc_num_slaves.data(),
-      disp_recv_ghost_slaves.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
-      slave_to_ghost);
+  MPI_Neighbor_alltoallv(out_ghost_slaves.data(), num_send_slaves.data(),
+                         disp_send_ghost_slaves.data(),
+                         dolfinx::MPI::mpi_t<std::int64_t>,
+                         in_ghost_slaves.data(), inc_num_slaves.data(),
+                         disp_recv_ghost_slaves.data(),
+                         dolfinx::MPI::mpi_t<std::int64_t>, slave_to_ghost);
   std::vector<std::int32_t> in_ghost_offsets(disp_recv_ghost_slaves.back());
-  MPI_Neighbor_alltoallv(
-      out_ghost_offsets.data(), num_send_slaves.data(),
-      disp_send_ghost_slaves.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
-      in_ghost_offsets.data(), inc_num_slaves.data(),
-      disp_recv_ghost_slaves.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
-      slave_to_ghost);
+  MPI_Neighbor_alltoallv(out_ghost_offsets.data(), num_send_slaves.data(),
+                         disp_send_ghost_slaves.data(),
+                         dolfinx::MPI::mpi_t<std::int32_t>,
+                         in_ghost_offsets.data(), inc_num_slaves.data(),
+                         disp_recv_ghost_slaves.data(),
+                         dolfinx::MPI::mpi_t<std::int32_t>, slave_to_ghost);
 
   // Communicate size of communication of masters
   std::vector<int> inc_num_masters(src_ranks_ghost.size() + 1);
@@ -1666,25 +1671,25 @@ mpc_data<T> create_contact_inelastic_condition(
   std::partial_sum(num_send_masters.begin(), num_send_masters.end(),
                    disp_send_ghost_masters.begin() + 1);
   std::vector<std::int64_t> in_ghost_masters(disp_recv_ghost_masters.back());
-  MPI_Neighbor_alltoallv(
-      out_ghost_masters.data(), num_send_masters.data(),
-      disp_send_ghost_masters.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
-      in_ghost_masters.data(), inc_num_masters.data(),
-      disp_recv_ghost_masters.data(), dolfinx::MPI::mpi_type<std::int64_t>(),
-      slave_to_ghost);
+  MPI_Neighbor_alltoallv(out_ghost_masters.data(), num_send_masters.data(),
+                         disp_send_ghost_masters.data(),
+                         dolfinx::MPI::mpi_t<std::int64_t>,
+                         in_ghost_masters.data(), inc_num_masters.data(),
+                         disp_recv_ghost_masters.data(),
+                         dolfinx::MPI::mpi_t<std::int64_t>, slave_to_ghost);
   std::vector<T> in_ghost_coeffs(disp_recv_ghost_masters.back());
   MPI_Neighbor_alltoallv(out_ghost_coeffs.data(), num_send_masters.data(),
-                         disp_send_ghost_masters.data(),
-                         dolfinx::MPI::mpi_type<T>(), in_ghost_coeffs.data(),
-                         inc_num_masters.data(), disp_recv_ghost_masters.data(),
-                         dolfinx::MPI::mpi_type<T>(), slave_to_ghost);
+                         disp_send_ghost_masters.data(), dolfinx::MPI::mpi_t<T>,
+                         in_ghost_coeffs.data(), inc_num_masters.data(),
+                         disp_recv_ghost_masters.data(), dolfinx::MPI::mpi_t<T>,
+                         slave_to_ghost);
   std::vector<std::int32_t> in_ghost_owners(disp_recv_ghost_masters.back());
-  MPI_Neighbor_alltoallv(
-      out_ghost_owners.data(), num_send_masters.data(),
-      disp_send_ghost_masters.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
-      in_ghost_owners.data(), inc_num_masters.data(),
-      disp_recv_ghost_masters.data(), dolfinx::MPI::mpi_type<std::int32_t>(),
-      slave_to_ghost);
+  MPI_Neighbor_alltoallv(out_ghost_owners.data(), num_send_masters.data(),
+                         disp_send_ghost_masters.data(),
+                         dolfinx::MPI::mpi_t<std::int32_t>,
+                         in_ghost_owners.data(), inc_num_masters.data(),
+                         disp_recv_ghost_masters.data(),
+                         dolfinx::MPI::mpi_t<std::int32_t>, slave_to_ghost);
 
   int err3 = MPI_Comm_free(&slave_to_ghost);
   dolfinx::MPI::check_error(comm, err3);
@@ -1751,7 +1756,7 @@ mpc_data<T> create_contact_inelastic_condition(
       = map_dofs_global_to_local<U>(V, in_ghost_slaves);
   for (std::int32_t i = 0; i < num_ghost_slaves; i++)
   {
-    if (ghost_offsets[i+1]  - ghost_offsets[i] == 0)
+    if (ghost_offsets[i + 1] - ghost_offsets[i] == 0)
       continue; // Skip if no masters for ghost slave
     slaves.push_back(local_ghosts[i]);
     for (std::int32_t j = ghost_offsets[i]; j < ghost_offsets[i + 1]; j++)
