@@ -1,4 +1,4 @@
-// Copyright (C) 2021 Jorgen S. Dokken
+// Copyright (C) 2021-2025 Jorgen S. Dokken
 //
 // This file is part of DOLFINX_MPC
 //
@@ -26,8 +26,11 @@ namespace impl
 /// @param[in, out] b The vector to apply lifting to
 /// @param[in] active_entities Set of active entities (either cells, exterior
 /// facets or interior facets in their specified format)
+/// @param[in] active_entities0 The active entities for the rows of the matrix
+/// @param[in] active_entities1 The active entities for the columns of the
+/// matrix
 /// @param[in] dofmap0 The dofmap for the rows of the matrix
-/// @param[in] dofmap0 The dofmap for the columns of the matrix
+/// @param[in] dofmap1 The dofmap for the columns of the matrix
 /// @param[in] bc_values1 Array of Dirichlet condition values for dofs local to
 /// process
 /// @param[in] bc_markers1 Array indicating what dofs local to process is in a
@@ -42,25 +45,28 @@ namespace impl
 template <typename T, std::size_t estride, std::floating_point U>
 void lift_bc_entities(
     std::span<T> b, std::span<const std::int32_t> active_entities,
+    std::span<const std::int32_t> active_entities0,
+    std::span<const std::int32_t> active_entities1,
     const dolfinx::fem::DofMap& dofmap0, const dolfinx::fem::DofMap& dofmap1,
     std::span<const T> bc_values1, std::span<const std::int8_t> bc_markers1,
-    const dolfinx_mpc::MultiPointConstraint<T, U>& mpc1,
+    const dolfinx_mpc::MultiPointConstraint<T, U>& mpc0,
     const std::function<const std::int32_t(std::span<const std::int32_t>)>
         fetch_cells,
     const std::function<void(std::span<T>, std::span<T>, const int, const int,
-                             std::span<const std::int32_t>, std::size_t)>
+                             std::span<const std::int32_t>, std::int32_t,
+                             std::int32_t, std::size_t)>
         lift_local_vector)
 {
   const int bs0 = dofmap0.bs();
   const int bs1 = dofmap1.bs();
   // Get MPC data
   const std::shared_ptr<const dolfinx::graph::AdjacencyList<std::int32_t>>
-      masters = mpc1.masters();
+      masters = mpc0.masters();
   const std::shared_ptr<const dolfinx::graph::AdjacencyList<T>> coefficients
-      = mpc1.coefficients();
-  std::span<const std::int8_t> is_slave = mpc1.is_slave();
+      = mpc0.coefficients();
+  std::span<const std::int8_t> is_slave = mpc0.is_slave();
   const std::shared_ptr<const dolfinx::graph::AdjacencyList<std::int32_t>>
-      cell_to_slaves = mpc1.cell_to_slaves();
+      cell_to_slaves = mpc0.cell_to_slaves();
 
   const int num_dofs0 = dofmap0.map().extent(1);
   std::vector<T> be;
@@ -72,9 +78,13 @@ void lift_bc_entities(
   {
     auto entity = active_entities.subspan(e, estride);
     const std::int32_t cell = fetch_cells(entity);
+    const std::int32_t cell0
+        = fetch_cells(active_entities0.subspan(e, estride));
+    const std::int32_t cell1
+        = fetch_cells(active_entities1.subspan(e, estride));
     // Size data structure for assembly
-    auto dmap0 = dofmap0.cell_dofs(cell);
-    auto dmap1 = dofmap1.cell_dofs(cell);
+    auto dmap0 = dofmap0.cell_dofs(cell0);
+    auto dmap1 = dofmap1.cell_dofs(cell1);
     const int num_rows = bs0 * dmap0.size();
     const int num_cols = bs1 * dmap1.size();
     be.resize(num_rows);
@@ -101,9 +111,10 @@ void lift_bc_entities(
     // Lift into local element vector
     const std::span<T> _be(be);
     const std::span<T> _Ae(Ae);
-    lift_local_vector(_be, _Ae, num_rows, num_cols, entity, e / estride);
+    lift_local_vector(_be, _Ae, num_rows, num_cols, entity, cell0, cell1,
+                      e / estride);
     // Modify local element matrix if entity is connected to a slave cell
-    std::span<const std::int32_t> slaves = cell_to_slaves->links(cell);
+    std::span<const std::int32_t> slaves = cell_to_slaves->links(cell1);
 
     if (slaves.size() > 0)
     {
@@ -135,19 +146,19 @@ void lift_bc_entities(
 /// @param[in] bcs List of boundary conditions
 /// @param[in] x0 The function to subtract
 /// @param[in] scale Scale of lifting
-/// @param[in] mpc1 The multi point constraints
+/// @param[in] mpc0 The multi point constraints
 template <typename T, std::floating_point U>
 void apply_lifting(
     std::span<T> b, const std::shared_ptr<const dolfinx::fem::Form<T>> a,
     const std::vector<std::shared_ptr<const dolfinx::fem::DirichletBC<T>>>& bcs,
     const std::span<const T>& x0, T scale,
-    const std::shared_ptr<const dolfinx_mpc::MultiPointConstraint<T, U>>& mpc1)
+    const std::shared_ptr<const dolfinx_mpc::MultiPointConstraint<T, U>>& mpc0)
 {
   const std::vector<T> constants = pack_constants(*a);
   auto coeff_vec = dolfinx::fem::allocate_coefficient_storage(*a);
   dolfinx::fem::pack_coefficients(*a, coeff_vec);
   auto coefficients = dolfinx::fem::make_coefficients_span(coeff_vec);
-  std::span<const std::int8_t> is_slave = mpc1->is_slave();
+  std::span<const std::int8_t> is_slave = mpc0->is_slave();
 
   // Create 1D arrays of bc values and bc indicator
   std::vector<std::int8_t> bc_markers1;
@@ -174,15 +185,15 @@ void apply_lifting(
   const int bs0 = a->function_spaces()[0]->dofmap()->bs();
   auto element0 = a->function_spaces()[0]->element();
 
-  const bool needs_transformation_data
-      = element0->needs_dof_transformations()
-        or element1->needs_dof_transformations()
-        or a->needs_facet_permutations();
-
-  auto mesh = a->function_spaces()[0]->mesh();
+  auto mesh0 = a->function_spaces()[0]->mesh();
+  assert(mesh0);
+  auto mesh1 = a->function_spaces()[1]->mesh();
+  assert(mesh1);
 
   // Prepare cell geometry
 
+  auto mesh = a->mesh();
+  assert(mesh);
   MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
       const std::int32_t,
       MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
@@ -192,11 +203,19 @@ void apply_lifting(
   const std::size_t num_dofs_g = x_dofmap.extent(1);
   std::vector<U> coordinate_dofs(3 * num_dofs_g);
 
-  std::span<const std::uint32_t> cell_info;
+  std::span<const std::uint32_t> cell_info0;
+  std::span<const std::uint32_t> cell_info1;
+  const bool needs_transformation_data
+      = element0->needs_dof_transformations()
+        or element1->needs_dof_transformations()
+        or a->needs_facet_permutations();
+
   if (needs_transformation_data)
   {
-    mesh->topology_mutable()->create_entity_permutations();
-    cell_info = std::span(mesh->topology()->get_cell_permutation_info());
+    mesh0->topology_mutable()->create_entity_permutations();
+    cell_info0 = std::span(mesh0->topology()->get_cell_permutation_info());
+    mesh1->topology_mutable()->create_entity_permutations();
+    cell_info1 = std::span(mesh1->topology()->get_cell_permutation_info());
   }
 
   // Get dof-transformations for the element matrix
@@ -213,164 +232,167 @@ void apply_lifting(
           dolfinx::fem::doftransform::transpose);
 
   // Loop over cell integrals and lift bc
-  if (a->num_integrals(dolfinx::fem::IntegralType::cell) > 0)
+
+  const auto fetch_cells
+      = [&](std::span<const std::int32_t> entity) { return entity.front(); };
+  for (int i = 0; i < a->num_integrals(dolfinx::fem::IntegralType::cell, 0);
+       ++i)
   {
-    const auto fetch_cells
-        = [&](std::span<const std::int32_t> entity) { return entity.front(); };
-    for (int i : a->integral_ids(dolfinx::fem::IntegralType::cell))
+    const auto& coeffs = coefficients.at({dolfinx::fem::IntegralType::cell, i});
+    const auto& kernel = a->kernel(dolfinx::fem::IntegralType::cell, i, 0);
+
+    // Function that lift bcs for cell kernels
+    const auto lift_bcs_cell
+        = [&](std::span<T> be, std::span<T> Ae, std::int32_t num_rows,
+              std::int32_t num_cols, std::span<const std::int32_t> entity,
+              std::int32_t cell0, std::int32_t cell1, std::size_t index)
     {
-      const auto& coeffs
-          = coefficients.at({dolfinx::fem::IntegralType::cell, i});
-      const auto& kernel = a->kernel(dolfinx::fem::IntegralType::cell, i);
+      auto cell = entity.front();
 
-      // Function that lift bcs for cell kernels
-      const auto lift_bcs_cell
-          = [&](std::span<T> be, std::span<T> Ae, std::int32_t num_rows,
-                std::int32_t num_cols, std::span<const std::int32_t> entity,
-                std::size_t index)
+      // Fetch the coordinates of the cell
+      auto x_dofs = MDSPAN_IMPL_STANDARD_NAMESPACE::submdspan(
+          x_dofmap, cell, MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
+      for (std::size_t i = 0; i < x_dofs.size(); ++i)
       {
-        auto cell = entity.front();
+        std::ranges::copy_n(std::next(x_g.begin(), 3 * x_dofs[i]), 3,
+                            std::next(coordinate_dofs.begin(), 3 * i));
+      }
 
-        // Fetch the coordinates of the cell
-        auto x_dofs = MDSPAN_IMPL_STANDARD_NAMESPACE::submdspan(
-            x_dofmap, cell, MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
-        for (std::size_t i = 0; i < x_dofs.size(); ++i)
+      // Tabulate tensor
+      std::ranges::fill(Ae, 0);
+      kernel(Ae.data(), coeffs.first.data() + index * coeffs.second,
+             constants.data(), coordinate_dofs.data(), nullptr, nullptr,
+             nullptr);
+      dof_transform(Ae, cell_info0, cell0, num_cols);
+      dof_transform_to_transpose(Ae, cell_info1, cell1, num_rows);
+
+      auto dmap1 = dofmap1->cell_dofs(cell1);
+      std::ranges::fill(be, 0);
+      for (std::size_t j = 0; j < dmap1.size(); ++j)
+      {
+        for (std::int32_t k = 0; k < bs1; k++)
         {
-          std::ranges::copy_n(std::next(x_g.begin(), 3 * x_dofs[i]), 3,
-                              std::next(coordinate_dofs.begin(), 3 * i));
-        }
-
-        // Tabulate tensor
-        std::ranges::fill(Ae, 0);
-        kernel(Ae.data(), coeffs.first.data() + index * coeffs.second,
-               constants.data(), coordinate_dofs.data(), nullptr, nullptr);
-        dof_transform(Ae, cell_info, cell, num_cols);
-        dof_transform_to_transpose(Ae, cell_info, cell, num_rows);
-
-        auto dmap1 = dofmap1->cell_dofs(cell);
-        std::ranges::fill(be, 0);
-        for (std::size_t j = 0; j < dmap1.size(); ++j)
-        {
-          for (std::int32_t k = 0; k < bs1; k++)
+          const std::int32_t jj = bs1 * dmap1[j] + k;
+          assert(jj < (int)bc_markers1.size());
+          // Add this in once we have interface for rhs of MPCs
+          // MPCs overwrite Dirichlet conditions
+          // if (is_slave[jj])
+          // {
+          //   // Lift MPC values
+          //   const T val = mpc_consts[jj];
+          //   for (int m = 0; m < num_rows; ++m)
+          //     be[m] -= Ae[m * num_cols + bs1 * j + k] * val;
+          // }
+          // else
+          if (bc_markers1[jj])
           {
-            const std::int32_t jj = bs1 * dmap1[j] + k;
-            assert(jj < (int)bc_markers1.size());
-            // Add this in once we have interface for rhs of MPCs
-            // MPCs overwrite Dirichlet conditions
-            // if (is_slave[jj])
-            // {
-            //   // Lift MPC values
-            //   const T val = mpc_consts[jj];
-            //   for (int m = 0; m < num_rows; ++m)
-            //     be[m] -= Ae[m * num_cols + bs1 * j + k] * val;
-            // }
-            // else
-            if (bc_markers1[jj])
-            {
-              const T bc = bc_values1[jj];
-              const T _x0 = x0.empty() ? 0.0 : x0[jj];
-              for (int m = 0; m < num_rows; ++m)
-                be[m] -= Ae[m * num_cols + bs1 * j + k] * scale * (bc - _x0);
-            }
+            const T bc = bc_values1[jj];
+            const T _x0 = x0.empty() ? 0.0 : x0[jj];
+            for (int m = 0; m < num_rows; ++m)
+              be[m] -= Ae[m * num_cols + bs1 * j + k] * scale * (bc - _x0);
           }
         }
-      };
-      // Assemble over all active cells
-      std::span<const std::int32_t> cells
-          = a->domain(dolfinx::fem::IntegralType::cell, i);
-      lift_bc_entities<T, 1>(b, cells, *dofmap0, *dofmap1,
-                             std::span<const T>(bc_values1),
-                             std::span<const std::int8_t>(bc_markers1), *mpc1,
-                             fetch_cells, lift_bcs_cell);
-    }
+      }
+    };
+    // Assemble over all active cells
+    std::span<const std::int32_t> cells
+        = a->domain(dolfinx::fem::IntegralType::cell, i, 0);
+    std::span<const std::int32_t> active_cells0
+        = a->domain_arg(dolfinx::fem::IntegralType::cell, 0, i, 0);
+    std::span<const std::int32_t> active_cells1
+        = a->domain_arg(dolfinx::fem::IntegralType::cell, 1, i, 0);
+    lift_bc_entities<T, 1>(b, cells, active_cells0, active_cells1, *dofmap0,
+                           *dofmap1, std::span<const T>(bc_values1),
+                           std::span<const std::int8_t>(bc_markers1), *mpc0,
+                           fetch_cells, lift_bcs_cell);
   }
 
-  // Prepare permutations for exterior and interior facet integrals
-  if (a->num_integrals(dolfinx::fem::IntegralType::exterior_facet) > 0)
+  // Get number of cells per facet to be able to get the facet permutation
+  for (int i = 0;
+       i < a->num_integrals(dolfinx::fem::IntegralType::exterior_facet, 0); ++i)
   {
-    // Create lambda function fetching cell index from exterior facet entity
-    const auto fetch_cell
-        = [&](std::span<const std::int32_t> entity) { return entity.front(); };
+    const auto& coeffs
+        = coefficients.at({dolfinx::fem::IntegralType::exterior_facet, i});
+    const auto& kernel
+        = a->kernel(dolfinx::fem::IntegralType::exterior_facet, i, 0);
 
-    // Get number of cells per facet to be able to get the facet permutation
-    const int tdim = mesh->topology()->dim();
-    for (int i : a->integral_ids(dolfinx::fem::IntegralType::exterior_facet))
+    /// Assemble local exterior facet kernels into a vector
+    /// @param[in] be The local element vector
+    /// @param[in] entity The entity, given as a cell index and the local
+    /// index relative to the cell
+    /// @param[in] index The index of the facet in the active_facets (To fetch
+    /// the appropriate coefficients)
+    const auto lift_bc_exterior_facet
+        = [&](std::span<T> be, std::span<T> Ae, int num_rows, int num_cols,
+              std::span<const std::int32_t> entity, std::int32_t cell0,
+              std::int32_t cell1, std::size_t index)
     {
-      const auto& coeffs
-          = coefficients.at({dolfinx::fem::IntegralType::exterior_facet, i});
-      const auto& kernel
-          = a->kernel(dolfinx::fem::IntegralType::exterior_facet, i);
+      // Fetch the coordiantes of the cell
+      const std::int32_t cell = entity[0];
+      const int local_facet = entity[1];
 
-      /// Assemble local exterior facet kernels into a vector
-      /// @param[in] be The local element vector
-      /// @param[in] entity The entity, given as a cell index and the local
-      /// index relative to the cell
-      /// @param[in] index The index of the facet in the active_facets (To fetch
-      /// the appropriate coefficients)
-      const auto lift_bc_exterior_facet
-          = [&](std::span<T> be, std::span<T> Ae, int num_rows, int num_cols,
-                std::span<const std::int32_t> entity, std::size_t index)
+      // Fetch the coordinates of the cell
+      auto x_dofs = MDSPAN_IMPL_STANDARD_NAMESPACE::submdspan(
+          x_dofmap, cell, MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
+      for (std::size_t i = 0; i < x_dofs.size(); ++i)
       {
-        // Fetch the coordiantes of the cell
-        const std::int32_t cell = entity[0];
-        const int local_facet = entity[1];
+        std::ranges::copy_n(std::next(x_g.begin(), 3 * x_dofs[i]), 3,
+                            std::next(coordinate_dofs.begin(), 3 * i));
+      }
 
-        // Fetch the coordinates of the cell
-        auto x_dofs = MDSPAN_IMPL_STANDARD_NAMESPACE::submdspan(
-            x_dofmap, cell, MDSPAN_IMPL_STANDARD_NAMESPACE::full_extent);
-        for (std::size_t i = 0; i < x_dofs.size(); ++i)
+      // Tabulate tensor
+      std::ranges::fill(Ae, 0);
+      kernel(Ae.data(), coeffs.first.data() + index * coeffs.second,
+             constants.data(), coordinate_dofs.data(), &local_facet, nullptr,
+             nullptr);
+      dof_transform(Ae, cell_info0, cell0, num_cols);
+      dof_transform_to_transpose(Ae, cell_info1, cell1, num_rows);
+
+      auto dmap1 = dofmap1->cell_dofs(cell1);
+      std::ranges::fill(be, 0);
+      for (std::size_t j = 0; j < dmap1.size(); ++j)
+      {
+        for (std::int32_t k = 0; k < bs1; k++)
         {
-          std::ranges::copy_n(std::next(x_g.begin(), 3 * x_dofs[i]), 3,
-                              std::next(coordinate_dofs.begin(), 3 * i));
-        }
+          const std::int32_t jj = bs1 * dmap1[j] + k;
+          assert(jj < (int)bc_markers1.size());
+          // Add this in once we have interface for rhs of MPCs
+          // MPCs overwrite Dirichlet conditions
+          // if (is_slave[jj])
+          // {
+          //   // Lift MPC values
+          //   const T val = mpc_consts[jj];
+          //   for (int m = 0; m < num_rows; ++m)
+          //     be[m] -= Ae[m * num_cols + bs1 * j + k] * val;
+          // }
+          // else
 
-        // Tabulate tensor
-        std::ranges::fill(Ae, 0);
-        kernel(Ae.data(), coeffs.first.data() + index * coeffs.second,
-               constants.data(), coordinate_dofs.data(), &local_facet, nullptr);
-        dof_transform(Ae, cell_info, cell, num_cols);
-        dof_transform_to_transpose(Ae, cell_info, cell, num_rows);
-
-        auto dmap1 = dofmap1->cell_dofs(cell);
-        std::ranges::fill(be, 0);
-        for (std::size_t j = 0; j < dmap1.size(); ++j)
-        {
-          for (std::int32_t k = 0; k < bs1; k++)
+          if (bc_markers1[jj])
           {
-            const std::int32_t jj = bs1 * dmap1[j] + k;
-            assert(jj < (int)bc_markers1.size());
-            // Add this in once we have interface for rhs of MPCs
-            // MPCs overwrite Dirichlet conditions
-            // if (is_slave[jj])
-            // {
-            //   // Lift MPC values
-            //   const T val = mpc_consts[jj];
-            //   for (int m = 0; m < num_rows; ++m)
-            //     be[m] -= Ae[m * num_cols + bs1 * j + k] * val;
-            // }
-            // else
-
-            if (bc_markers1[jj])
-            {
-              const T bc = bc_values1[jj];
-              const T _x0 = x0.empty() ? 0.0 : x0[jj];
-              for (int m = 0; m < num_rows; ++m)
-                be[m] -= Ae[m * num_cols + bs1 * j + k] * scale * (bc - _x0);
-            }
+            const T bc = bc_values1[jj];
+            const T _x0 = x0.empty() ? 0.0 : x0[jj];
+            for (int m = 0; m < num_rows; ++m)
+              be[m] -= Ae[m * num_cols + bs1 * j + k] * scale * (bc - _x0);
           }
         }
-      };
+      }
+    };
 
-      // Assemble over all active cells
-      std::span<const std::int32_t> active_facets
-          = a->domain(dolfinx::fem::IntegralType::exterior_facet, i);
-      impl::lift_bc_entities<T, 2>(b, active_facets, *dofmap0, *dofmap1,
-                                   bc_values1, bc_markers1, *mpc1, fetch_cell,
-                                   lift_bc_exterior_facet);
-    }
+    // Assemble over all active cells
+    std::span<const std::int32_t> active_facets
+        = a->domain(dolfinx::fem::IntegralType::exterior_facet, i, 0);
+    std::span<const std::int32_t> active_facets0
+        = a->domain_arg(dolfinx::fem::IntegralType::exterior_facet, 0, i, 0);
+    std::span<const std::int32_t> active_facets1
+        = a->domain_arg(dolfinx::fem::IntegralType::exterior_facet, 1, i, 0);
+
+    impl::lift_bc_entities<T, 2>(
+        b, active_facets, active_facets0, active_facets1, *dofmap0, *dofmap1,
+        bc_values1, bc_markers1, *mpc0, fetch_cells, lift_bc_exterior_facet);
   }
-  if (a->num_integrals(dolfinx::fem::IntegralType::interior_facet) > 0)
+  if (a->num_integrals(dolfinx::fem::IntegralType::interior_facet, 0) > 0)
   {
+
     throw std::runtime_error(
         "Interior facet integrals currently not supported");
 
@@ -432,16 +454,24 @@ void apply_lifting(
     throw std::runtime_error(
         "Mismatch in size between a and bcs in assembler.");
   }
+  // If all forms are null, there is nothing to do
+  if (std::ranges::all_of(a, [](auto ai) { return !ai; }))
+    return;
+
   for (std::size_t j = 0; j < a.size(); ++j)
   {
-    if (x0.empty())
+    if (a[j] and !bcs1[j].empty())
     {
-      impl::apply_lifting<double>(b, a[j], bcs1[j], std::span<const double>(),
-                                  scale, mpc);
-    }
-    else
-    {
-      impl::apply_lifting<double>(b, a[j], bcs1[j], x0[j], scale, mpc);
+
+      if (x0.empty())
+      {
+        impl::apply_lifting<double>(b, a[j], bcs1[j], std::span<const double>(),
+                                    scale, mpc);
+      }
+      else
+      {
+        impl::apply_lifting<double>(b, a[j], bcs1[j], x0[j], scale, mpc);
+      }
     }
   }
 }
@@ -491,16 +521,19 @@ void apply_lifting(
   }
   for (std::size_t j = 0; j < a.size(); ++j)
   {
-    if (x0.empty())
+    if (a[j] and !bcs1[j].empty())
     {
-      impl::apply_lifting<std::complex<double>>(
-          b, a[j], bcs1[j], std::span<const std::complex<double>>(), scale,
-          mpc);
-    }
-    else
-    {
-      impl::apply_lifting<std::complex<double>>(b, a[j], bcs1[j], x0[j], scale,
-                                                mpc);
+      if (x0.empty())
+      {
+        impl::apply_lifting<std::complex<double>>(
+            b, a[j], bcs1[j], std::span<const std::complex<double>>(), scale,
+            mpc);
+      }
+      else
+      {
+        impl::apply_lifting<std::complex<double>>(b, a[j], bcs1[j], x0[j],
+                                                  scale, mpc);
+      }
     }
   }
 }
@@ -547,14 +580,17 @@ void apply_lifting(
   }
   for (std::size_t j = 0; j < a.size(); ++j)
   {
-    if (x0.empty())
+    if (a[j] and !bcs1[j].empty())
     {
-      impl::apply_lifting<float>(b, a[j], bcs1[j], std::span<const float>(),
-                                 scale, mpc);
-    }
-    else
-    {
-      impl::apply_lifting<float>(b, a[j], bcs1[j], x0[j], scale, mpc);
+      if (x0.empty())
+      {
+        impl::apply_lifting<float>(b, a[j], bcs1[j], std::span<const float>(),
+                                   scale, mpc);
+      }
+      else
+      {
+        impl::apply_lifting<float>(b, a[j], bcs1[j], x0[j], scale, mpc);
+      }
     }
   }
 }
@@ -605,15 +641,19 @@ void apply_lifting(
   }
   for (std::size_t j = 0; j < a.size(); ++j)
   {
-    if (x0.empty())
+    if (a[j] and !bcs1[j].empty())
     {
-      impl::apply_lifting<std::complex<float>>(
-          b, a[j], bcs1[j], std::span<const std::complex<float>>(), scale, mpc);
-    }
-    else
-    {
-      impl::apply_lifting<std::complex<float>>(b, a[j], bcs1[j], x0[j], scale,
-                                               mpc);
+      if (x0.empty())
+      {
+        impl::apply_lifting<std::complex<float>>(
+            b, a[j], bcs1[j], std::span<const std::complex<float>>(), scale,
+            mpc);
+      }
+      else
+      {
+        impl::apply_lifting<std::complex<float>>(b, a[j], bcs1[j], x0[j], scale,
+                                                 mpc);
+      }
     }
   }
 }
