@@ -11,12 +11,11 @@ import dolfinx.fem as fem
 import numpy as np
 import pytest
 import ufl
-from dolfinx.common import Timer, TimingType, list_timings
+from dolfinx.common import Timer, list_timings
 from dolfinx.mesh import CellType, create_unit_square
 
 import dolfinx_mpc
 import dolfinx_mpc.utils
-from dolfinx_mpc.utils import get_assemblers  # noqa: F401
 
 root = 0
 
@@ -100,4 +99,4 @@ def test_slave_on_same_cell(master_point, degree, celltype, get_assemblers):  #
         A_org.assemble()
         dolfinx_mpc.utils.compare_mpc_lhs(A_org, A_mpc, mpc)
 
-    list_timings(mesh.comm, [TimingType.wall])
+    list_timings(mesh.comm)
